@@ -14,7 +14,7 @@ from math import factorial as _factorial
 import numpy as _np
 
 
-__version__ = '0.1.0'
+__version__ = '0.2.0'
 
 
 class Monomial:
@@ -23,27 +23,33 @@ class Monomial:
     def __init__(self, segments, grid):
         r"""Piecewise polynomial curve using monomial basis.
 
-        Arbitrary degree, arbitrary dimension.
+        See :ref:`/euclidean/polynomials.ipynb`.
+
+        Coefficients can have arbitrary dimension.
+        An arbitrary polynomial degree :math:`d` can be used by specifying
+        :math:`d + 1` coefficients per segment.
+        The :math:`i`-th segment is evaluated using this equation:
 
         .. math::
 
-            \boldsymbol{p}_i(t) = \sum_{k=0}^n
-                \boldsymbol{a}_k \left(\frac{t - t_i}{t_{i+1} - t_i}\right)^k
+            \boldsymbol{p}_i(t) = \sum_{k=0}^d
+                \boldsymbol{a}_{i,k} \left(\frac{t - t_i}{t_{i+1} - t_i}\right)^k
                 \text{ for } t_i \leq t < t_{i+1}
 
-        Similar to https://docs.scipy.org/doc/scipy/reference/generated/scipy.interpolate.PPoly.html,
-        which states:
+        This is similar to `scipy.interpolate.PPoly`, which states:
 
             "High-order polynomials in the power basis can be numerically
             unstable.  Precision problems can start to appear for orders
             larger than 20-30."
 
+        This shouldn't be a problem since most commonly splines of degree 3
+        (i.e. cubic splines) are used.
 
         :param segments: Sequence of polynomial segments.
-            Each segment contains coefficients for the monomial basis
-            (in order of decreasing degree).
+            Each segment :math:`\boldsymbol{a}_i` contains coefficients
+            for the monomial basis (in order of decreasing degree).
             Different segments can have different polynomial degree.
-        :param grid: Sequence of parameter values corresponding to
+        :param grid: Sequence of parameter values :math:`t_i` corresponding to
             segment boundaries.  Must be strictly increasing.
 
         """
@@ -54,7 +60,7 @@ class Monomial:
         self.grid = list(grid)
 
     def evaluate(self, t, n=0):
-        """Get value (or *n*-th derivative) at given parameter value(s)."""
+        """Get value (or *n*-th derivative) at given parameter value(s) *t*."""
         if not _np.isscalar(t):
             return _np.array([self.evaluate(time, n) for time in t])
 
@@ -95,6 +101,8 @@ class Bernstein:
     def __init__(self, segments, grid=None):
         """Piecewise Bézier curve using Bernstein basis.
 
+        See :ref:`/euclidean/bezier.ipynb`.
+
         :param segments: Sequence of segments,
             each one consisting of multiple Bézier control points.
             Different segments can have different numbers of control points
@@ -229,13 +237,13 @@ def _natural_tangent(vertices, times, tangent):
     Given 2 points and one tangent, this returns the tangent for the
     other side that results from the second derivative being zero.
 
-    See :ref:`end-conditions-natural.ipynb`.
+    See euclidan/end-conditions-natural.ipynb.
 
     """
     x0, x1 = _np.asarray(vertices)
     t0, t1 = times
     delta = t1 - t0
-    return (3 * x1 - 3 * x0 - delta * tangent) / (2 * delta)
+    return 3 * (x1 - x0) / (2 * delta) - tangent / 2
 
 
 class CubicHermite(Monomial):
@@ -298,8 +306,9 @@ class CatmullRom(CubicHermite):
         t_1, t0, t1 = times
         delta_1 = t0 - t_1
         delta0 = t1 - t0
-        return ((delta0**2 * (x0 - x_1) + delta_1**2 * (x1 - x0)) /
-                (delta0 * delta_1 * (delta0 + delta_1)))
+        v_1 = (x0 - x_1) / delta_1
+        v0 = (x1 - x0) / delta0
+        return (delta0 * v_1 + delta_1 * v0) / (delta0 + delta_1)
 
     def __init__(self, vertices, grid=None, *, alpha=None,
                  endconditions='natural'):
@@ -360,16 +369,12 @@ class KochanekBartels(CubicHermite):
         b = (1 - T) * (1 - C) * (1 - B)
         c = (1 - T) * (1 - C) * (1 + B)
         d = (1 - T) * (1 + C) * (1 - B)
-        incoming = (
-            c * (t1 - t0)**2 * (x0 - x_1) + d * (t0 - t_1)**2 * (x1 - x0)
-        ) / (
-            (t1 - t0) * (t0 - t_1) * (t1 - t_1)
-        )
-        outgoing = (
-            a * (t1 - t0)**2 * (x0 - x_1) + b * (t0 - t_1)**2 * (x1 - x0)
-        ) / (
-            (t1 - t0) * (t0 - t_1) * (t1 - t_1)
-        )
+        delta_1 = t0 - t_1
+        delta0 = t1 - t0
+        v_1 = (x0 - x_1) / delta_1
+        v0 = (x1 - x0) / delta0
+        incoming = (c * delta0 * v_1 + d * delta_1 * v0) / (delta_1 + delta0)
+        outgoing = (a * delta0 * v_1 + b * delta_1 * v0) / (delta_1 + delta0)
         return incoming, outgoing
 
     def __init__(self, vertices, grid=None, *, tcb=(0, 0, 0), alpha=None,
