@@ -10,6 +10,10 @@ class Quaternion:
 
     This is the base class for the more relevant `UnitQuaternion` class.
 
+    See the `notebook about quaternions`__.
+
+    __ ../rotation/quaternions.ipynb
+
     """
 
     __slots__ = '_scalar', '_vector'
@@ -23,10 +27,12 @@ class Quaternion:
 
     @property
     def scalar(self):
+        """The scalar part (a.k.a. real part) of the quaternion."""
         return self._scalar
 
     @property
     def vector(self):
+        """The vector part (a.k.a. imaginary part) of the quaternion."""
         return self._vector
 
     def __repr__(self):
@@ -57,7 +63,8 @@ class Quaternion:
             vector=(s_x - o_x, s_y - o_y, s_z - o_z))
 
     def __mul__(self, other):
-        if isinstance(self, UnitQuaternion) and isinstance(other, UnitQuaternion):
+        if isinstance(self, UnitQuaternion) and \
+                isinstance(other, UnitQuaternion):
             result_type = UnitQuaternion
         elif isinstance(other, Quaternion):
             result_type = Quaternion
@@ -90,10 +97,12 @@ class Quaternion:
         return Quaternion.__new__(type(self), -self.scalar, (-x, -y, -z))
 
     def conjugate(self):
+        """Return quaternion with same `scalar` part, negated `vector` part."""
         x, y, z = self.vector
         return Quaternion.__new__(type(self), self.scalar, (-x, -y, -z))
 
-    def normalize(self):
+    def normalized(self):
+        """Return quaternion with same 4D direction but unit `norm`."""
         norm = self.norm
         x, y, z, w = self.xyzw
         return UnitQuaternion.from_unit_xyzw(
@@ -107,7 +116,7 @@ class Quaternion:
 
         Note that this is different from the quaternion multiplication
         (``q1 * q2``), which produces another quaternion
-        (and is non-commutative).
+        (and is noncommutative).
 
         """
         # NB: math.prod() is available since Python 3.8
@@ -116,22 +125,31 @@ class Quaternion:
 
     @property
     def norm(self):
+        """Length of the quaternion in 4D space."""
         x, y, z, w = self.xyzw
         return _math.sqrt(x**2 + y**2 + z**2 + w**2)
 
     @property
     def xyzw(self):
+        """Components of the quaternion, `scalar` last."""
         x, y, z = self.vector
         return x, y, z, self.scalar
 
     @property
     def wxyz(self):
+        """Components of the quaternion, `scalar` first."""
         x, y, z = self.vector
         return self.scalar, x, y, z
 
 
 class UnitQuaternion(Quaternion):
-    """Unit quaternion."""
+    """Unit quaternion.
+
+    See the `section about unit quaternions`__.
+
+    __ ../rotation/quaternions.ipynb#Unit-Quaternions
+
+    """
 
     __slots__ = ()
 
@@ -140,7 +158,7 @@ class UnitQuaternion(Quaternion):
 
     @classmethod
     def from_axis_angle(cls, axis, angle):
-        """Create a unit quaternion from a rotation axis and angle.
+        """Create a unit quaternion from a rotation `axis` and `angle`.
 
         :param axis: Three-component rotation axis. This will be normalized.
         :param angle: Rotation angle in radians.
@@ -155,7 +173,7 @@ class UnitQuaternion(Quaternion):
     def from_unit_xyzw(cls, xyzw):
         """Create a unit quaternion from another unit quaternion.
 
-        :param xyzw: Components of a unit quaternion (scalar last).
+        :param xyzw: Components of a unit quaternion (`scalar` last).
             This will *not* be normalized, it must already have unit length.
 
         """
@@ -178,13 +196,12 @@ class UnitQuaternion(Quaternion):
 
     @classmethod
     def exp_map(cls, value):
-        """Exponential map from :math:`R^3` to quaternions.
+        """Exponential map from :math:`R^3` to unit quaternions.
 
         This is the inverse operation to `log_map()`.
 
         :param value: Element of the tangent space at the quaternion identity.
-        :param type: 3-tuple
-        :returns: Corresponding unit quaternion.
+        :type value: 3-tuple
 
         """
         x, y, z = value
@@ -196,7 +213,7 @@ class UnitQuaternion(Quaternion):
         return super().__new__(cls, _math.cos(norm), (s * x, s * y, s * z))
 
     def log_map(self):
-        """Logarithmic map from quaternions to :math:`R^3`.
+        """Logarithmic map from unit quaternions to :math:`R^3`.
 
         :returns: Corresponding vector in the tangent space at the
             quaternion identity.
@@ -209,13 +226,17 @@ class UnitQuaternion(Quaternion):
 
     @property
     def axis(self):
+        """The (normalized) rotation axis."""
         assert self.scalar <= 1
         # NB: This is the same as sqrt(x**2 + y**2 + z**2)
         norm = _math.sqrt(1 - self.scalar**2)
-        return _np.true_divide(self.vector, norm)
+        # Avoid warning for (1, (0, 0, 0)) -> (NaN, NaN, NaN):
+        with _np.errstate(invalid='ignore'):
+            return _np.true_divide(self.vector, norm)
 
     @property
     def angle(self):
+        """The rotation angle in radians."""
         return 2 * _math.acos(self.scalar)
 
     def rotation_to(self, other):
@@ -223,16 +244,28 @@ class UnitQuaternion(Quaternion):
 
         See :ref:`/rotation/quaternions.ipynb#Relative-Rotation-(Global-Frame-of-Reference)`.
 
+        :param other: Target rotation.
+        :type other: UnitQuaternion
+
+        :returns: Relative rotation (as `UnitQuaternion`).
+
         """
         return other * self.inverse()
 
     def rotate_vector(self, v):
+        """Apply rotation to a 3D vector.
+
+        :param v: A vector in :math:`R^3`.
+        :type v: 3-tuple
+        :returns: The rotated vector.
+
+        """
         rotated = self * Quaternion(0, v) * self.inverse()
         return rotated.vector
 
 
 def slerp(one, two, t):
-    """Spherical linear interpolation.
+    """Spherical Linear intERPolation.
 
     See :ref:`/rotation/slerp.ipynb`.
 
@@ -245,7 +278,7 @@ def slerp(one, two, t):
 
 
 def canonicalized(quaternions):
-    """Iterator adapter to ensure minimal angles between quaternions."""
+    """Iterator adapter to ensure minimal angles between *quaternions*."""
     p = UnitQuaternion.from_unit_xyzw((0, 0, 0, 1))
     for q in quaternions:
         if p.dot(q) < 0:
@@ -362,7 +395,7 @@ class DeCasteljau:
 
 
 def _reduce_de_casteljau(segment, t):
-    """Obtain two quaternions for the last step of De Castelau's algorithm.
+    """Obtain two quaternions for the last step of De Casteljau's algorithm.
 
     Repeatedly applies `slerp()` to neighboring control quaternions
     until only two are left.
@@ -392,24 +425,28 @@ class KochanekBartels(DeCasteljau):
 
         q_in = q0 * q_1.inverse()
         q_out = q1 * q0.inverse()
-        v_in = q_in.log_map() / (t0 - t_1)
-        v_out = q_out.log_map() / (t1 - t0)
+        rho_in = q_in.log_map() / (t0 - t_1)
+        rho_out = q_out.log_map() / (t1 - t0)
 
-        def v0(weight_in, weight_out):
+        def omega(weight_in, weight_out):
             return (
-                weight_in * (t1 - t0) * v_in +
-                weight_out * (t0 - t_1) * v_out
+                weight_in * (t1 - t0) * rho_in +
+                weight_out * (t0 - t_1) * rho_out
             ) / (t1 - t_1)
 
         return [
-            UnitQuaternion.exp_map(-v0(c, d) * (t0 - t_1) / 3) * q0,
-            UnitQuaternion.exp_map(v0(a, b) * (t1 - t0) / 3) * q0,
+            UnitQuaternion.exp_map(-omega(c, d) * (t0 - t_1) / 3) * q0,
+            UnitQuaternion.exp_map(omega(a, b) * (t1 - t0) / 3) * q0,
         ]
 
     def __init__(self, quaternions, grid=None, *, tcb=(0, 0, 0), alpha=None,
                  endconditions='natural'):
         """Kochanek--Bartels-like rotation spline.
 
+        See `the corresponding notebook`__ for details.
+
+        __ ../rotation/kochanek-bartels.ipynb
+
         :param quaternions: Sequence of rotations to be interpolated.
             The quaternions will be `canonicalized()`.
         :param grid: Sequence of parameter values.
@@ -490,6 +527,9 @@ class CatmullRom(KochanekBartels):
 
         This is just `KochanekBartels` without TCB values.
 
+        See :ref:`/rotation/catmull-rom-uniform.ipynb`
+        and :ref:`/rotation/catmull-rom-non-uniform.ipynb`.
+
         """
         super().__init__(
             quaternions, grid, tcb=(0, 0, 0),
@@ -559,7 +599,8 @@ def _check_endconditions(endconditions, quaternions, grid):
 
 def _end_control_quaternion(condition, quaternions):
     if condition == 'natural':
-        return _natural_control_quaternion(quaternions)
+        first, _, third = quaternions
+        return _natural_control_quaternion(first, third)
     elif _np.shape(condition) == _np.shape(quaternions[0]):
         #tangent = condition
         raise NotImplementedError('TODO')
@@ -567,13 +608,18 @@ def _end_control_quaternion(condition, quaternions):
         f'{condition!r} is not a valid start/end condition')
 
 
-def _natural_control_quaternion(quaternions):
-    """Return second control quaternion given the other three."""
-    outer, inner_control, inner = quaternions
-    return (
-        (inner_control * inner.inverse()) *
-        (inner * outer.inverse())
-    )**(1 / 2) * outer
+def _natural_control_quaternion(first, third):
+    """Natural end condition for "cubic" Bézier curves.
+
+    Returns the second control quaternion given the first and the third.
+
+    This can also be used for the end of the spline,
+    when counting the control quaternions from the end.
+
+    See rotation/end-conditions-natural.ipynb.
+
+    """
+    return first.rotation_to(third)**(1 / 2) * first
 
 
 class BarryGoldman:
